﻿Highcharts.setOptions({
	global: {
		useUTC: false
	}
});

var HOUR = 3600 * 1000, SIX_HOUR = 6 * 3600 * 1000, HALF_DAY = 12 * 3600 * 1000, DAY = 24 * 3600 * 1000, TWO_DAY = 48 * 3600 * 1000;
var ChartZoomLevelRaw = 0, ChartZoomLevelNormal = 1, ChartZoomLevelBand = 2;
var zoomLevel = ChartZoomLevelNormal;
var rawDataDates = [];
var pendingStartDate = null, pendingEndDate = null;
var statData = null;

function addRawDataPoint(dataPoint, prepend, cache, series) {
	if (prepend) {
		if (cache.length > 0) {
			if (cache[0][0] - dataPoint[0] > 5 * HOUR) {
				var nullPoint = [dataPoint[0] + 1000, null];
				cache.unshift(nullPoint); series.addPoint(nullPoint, false);
			}
		}
		cache.unshift(dataPoint);
	} else {
		if (cache.length > 0) {
			if (dataPoint[0] - cache[cache.length - 1][0] > 5 * HOUR) {
				var nullPoint = [dataPoint[0] - 1000, null];
				cache.push(nullPoint); series.addPoint(nullPoint, false);
			}
		}
		cache.push(dataPoint);
	}
	series.addPoint(dataPoint, false);
}

function clearRawData() {
	pendingEndDate = null;
	pendingStartDate = null;
	clearCachedRawData();
	rawDataDates.length = 0;
}
function  stringFromDate(unixtime){
	var date = new Date(unixtime);
	return (date.getMonth() + 1) + '/' + date.getDate() + '/' +  date.getFullYear();
}
var chart; 
var currentRange;

function loadDataTo(end, done) {
	var oldEndDate = pendingEndDate;
	pendingEndDate = end
			
	//console.log("loadDataTo " + oldEndDate.toString(), pendingEndDate.toString()));
	$loader.appendTo($body).css({ top: 100 });
	dataLoader(stringFromDate(oldEndDate+DAY), stringFromDate(pendingEndDate), function(days){			
		$loader.remove();
		if(days.length==0)return;			
		for(var i=0;i<days.length;i++){
			var day =days[i];
			rawDataDates.push(Date.parse(day.date));
		}
		addRawDataMultipleDays(days, false);
		done();
	});
}
function loadDataFrom(start){
	var oldStartDate = pendingStartDate;
	pendingStartDate = start;
			
	$loader.appendTo($body).css({ top: 100 });
	dataLoader(stringFromDate(pendingStartDate), stringFromDate(oldStartDate-DAY), function(days){
		$loader.remove();
		if(days.length==0)return;
		for(var i=0;i<days.length;i++){
			var day =days[days.length-i-1];
			rawDataDates.unshift(Date.parse(day.date));
		}
		addRawDataMultipleDays(days, true);
	});
}

var dataRange = {};

function enteredNormalLevel(range, done) {
	if (statData == null) {
		$loader.appendTo($body).css({ top: 100 });
		hourlyDataLoader(function (stats) {
			statData = stats;

			var nDays = statData.length;
			if (nDays == 0) { popup("No data available for any of these tags."); $loader.remove(); return; }
			dataRange.max = Date.parse(statData[0].date);
			dataRange.min = Date.parse(statData[nDays - 1].date);
			if (range == null) range = dataRange;

			if (range.max - range.min > 60 * DAY)
				zoomLevel = ChartZoomLevelBand;
			else
				zoomLevel = ChartZoomLevelNormal;

			for (var i = 1; i <= nDays; i++) {
				var day = statData[nDays - i];
				processOneDay(day, Date.parse(day.date), false);
			}
			updateChartType(zoomLevel);  // calls setData except raw

			done();
			$loader.remove();

		});
	} else {
		updateChartType(zoomLevel);  // calls setData except raw
		done();
	}
}
function enteredRawLevelWithRange(range, done) {
	if(pendingEndDate==null){
		clearRawData();
		pendingEndDate = range.min - HOUR * 36; pendingStartDate = pendingEndDate + DAY;
		loadDataTo(range.max + DAY, done);
	}
	else {				
		if (range.min - pendingEndDate > DAY || range.max - pendingStartDate < -DAY ) {
			clearRawData();
			pendingEndDate = range.min - HOUR * 36; pendingStartDate = pendingEndDate + DAY;
			loadDataTo(range.max + DAY, done);
		} else {
			loadCachedRawData();
			done();
		}
	}
}

function updatePan(c, range) {
	chart=c; currentRange= range;
	if (zoomLevel < ChartZoomLevelNormal){
		if(range.max>pendingEndDate) {
			loadDataTo(pendingEndDate + HOUR * 48, function () { });
			return true;
		}
			// if min is on the last minute of first day, load 2 more days.
		else if(range.min < pendingStartDate+DAY) {
			loadDataFrom(pendingStartDate-HOUR*48);
			return true;
		} 
	}
}
var statType = "temperature";
function loadFirstData() {
	var ti = window.localStorage != null ? localStorage["mytaglist.stats." + statType + ".ti"] : null;
	if (ti == null) ti = 71 * HOUR;
	if (ti != null) {
		dataSpanLoader(function (from, to) {
			dataRange = { min: from, max: to };					
			var range = { min: dataRange.max - ti, max: dataRange.max };
				
			zoomLevel = zoomLevelFromTI(ti);
			if (zoomLevel >= ChartZoomLevelNormal) {

				enteredNormalLevel(range, function () {   // this assigns dataRange
					chart.xAxis[0].setExtremes(range.min, range.max);
				});

			} else {   // raw range
				enteredRawLevelWithRange(range, function(){
					updateChartType(zoomLevel);  // calls setData except raw
					chart.xAxis[0].setExtremes(range.min, range.max);
				});					// start async loading if no data, otherwise translateRawData (populate rawSeries and call chart.setData)		
			}
		});
		return;
	}
	enteredNormalLevel(null, function () { });
}
function zoomLevelFromTI(ti){
	if (ti < 72 * HOUR)
		return ChartZoomLevelRaw;
	else if (ti < 60 * DAY)
		return ChartZoomLevelNormal;
	else
		return ChartZoomLevelBand;
}
function updateZoom(c, range, done) 
{
	chart = c; currentRange = range;
	var ti = (range.max - range.min);
	//if (ti < 0) return;

	if (window.localStorage != null) localStorage["mytaglist.stats." + statType + ".ti"] = ti;

	var oldZoomLevel = zoomLevel;
	zoomLevel = zoomLevelFromTI(ti);

	if (zoomLevel < ChartZoomLevelNormal && oldZoomLevel >= ChartZoomLevelNormal) {
		updateChartType(zoomLevel);  // calls setData except raw
		enteredRawLevelWithRange(range, done);					// start async loading if no data, otherwise translateRawData (populate rawSeries and call chart.setData)		
	}
	else if (zoomLevel > ChartZoomLevelRaw && oldZoomLevel <= ChartZoomLevelRaw)
	{
		enteredNormalLevel(range, done);
	}
	else if (oldZoomLevel != zoomLevel) {
		updateChartType(zoomLevel);  // calls setData except raw
		done();
	} else {
		done();
	}
}
function addRawDataMultipleDays(days, prepend) {
	for (var i = 0; i < days.length; i++) {
		var day = prepend ? days[days.length - i - 1] : days[i];
		processOneDay(day, Date.parse(day.date), prepend);
	}
	chart.redraw();
}
